
int cellWidth = 22;
int cellHeight = 50;
int bgColor = 15;

float factHeight;
float morningOffset;

int rowMargin = 180;
int colMargin = 80;

int numWeekdays = 7;
int numWeeks = 51; //use 51 to correct for missing days and end up with rectangular shape
String[] weekdayNames = {
  "Sunday", 
  "Monday", 
  "Tuesday", 
  "Wednesday", 
  "Thursday", 
  "Friday", 
  "Saturday"
};

String[] weekdayNamesShort = {
  "Su", 
  "Mo", 
  "Tu", 
  "We", 
  "Th", 
  "Fr", 
  "Sa"
};

float [][] dataMatrix = null;
int [][] dataAwake = null;
//int [][] dataEarliestCout = null;
//int [][] dataLatestCout = null;

int [][] dataWeekByAwake = null;
int [][] dataWeekdayByAwake = null;
int [][] dataAwakeByAwake = null;
int [][] dataEarliestCout = null;
int [][] dataLatestCout = null;

int [][] dataWeekdayByAwakeCentered = null;

int maxAwake;
int numRows, numCols;

PFont myFont1 = createFont( "Helvetica", 24, true);
PFont myFont2 = createFont( "Helvetica", 16, true);

Table myTable;

color colorWeekday[]     = {
  color(252, 187, 161, 100), //Sun
  color(252, 146, 114, 100), //Mon
  color(251, 106, 74, 100), //Tue
  color(222, 45, 38, 100), 
  color(251, 106, 74, 100), 
  color(252, 146, 114, 100), 
  color(252, 187, 161, 100) //Sat
};

//['rgb(254,229,217)','rgb(252,187,161)','rgb(252,146,114)','rgb(251,106,74)','rgb(222,45,38)','rgb(165,15,21)']
//['rgb(254,240,217)','rgb(253,204,138)','rgb(252,141,89)','rgb(215,48,31)']

void setup()
{
  //setting up a screen size 1400*500
  size( 1400, 500);
  colorMode(RGB);
  // set background to white
  background(255);
  //activate smoothing anti-aliasing functions between pixels
  smooth();
  // set framerate to 10 per second. NOrmal video is 30 or 60
  frameRate( 10);

  //load the awake data set
  myTable = loadTable( "awake_Week_weekday_matrix_2006.csv", "header");
  //header for col names  as follows:
  //barcode  deweyClass  weekday_num  week_num  awake  earliest_cout_hour  latest_cout_hour

  // assign these variables to contain the number of rows and columns from myTable
  numRows = myTable.getRowCount();
  numCols = myTable.getColumnCount();

  // initialize the 2D array with number of rows and columns
  dataMatrix = new float[numRows][numCols];

  // copy everything from table into a 2D array

  maxAwake = (int)dataMatrix[0][4]; //find max awake value [minutes] for scaling of colorbar

  for ( int i = 0; i< numRows; i++)          // a for loop where i is set to 0  and increments all the way to numRows by 1
  {
    for ( int j = 0; j< numCols; j++)// a for loop where j is set to 0  and increments all the way to numColumns by 1
    {
      dataMatrix[i][j] = myTable.getInt(i, j);      // copying the table integer value at mytable (i,j) position into the dataMatrix
      print( dataMatrix[i][j ] + " ");            // print out the value of dataMatrix
    }                  
    if ( dataMatrix[i][4] > maxAwake)      // this is an if statement which checks for the condition in brackets                                                         // if true, it executes the statements in brackets 
    {
      maxAwake = (int)dataMatrix[i][4];
    }
    println();      // switch to next line in the prompt, improves legibility
  }

  //  //correct weekday numbering scheme to starting from Saturday instead of Sunday:
  //  for ( int i = 0; i< numRows; i++)          // a for loop where i is set to 0  and increments all the way to numRows by 1
  //  {
  //      dataMatrix[i][2] ++;
  //      if (dataMatrix[i][2] == 7) dataMatrix[i][2] = 0;
  //  }


  //get sorted week - weekday matrix from getWeekdayWeek function
  println("getting sorted matrices...");
  //dataAwake = getWeekWeekday(dataMatrix, 4); // 4 is colIndex for awake
  //println("interpolating missing data...");
  //dataAwake = interpMissing(dataAwake);

  // could be replaced by hashmap to make avoid multiple sortings of the same rules
  println("getting sorted weekday matrix...");
  dataWeekdayByAwake = getColByAwake(dataMatrix, 2); // 2 weekday
  println("getting sorted week matrix...");
  dataWeekByAwake = getColByAwake(dataMatrix, 3); // 3 is colIndex for week
  println("getting sorted awake matrix...");
  dataAwakeByAwake = getColByAwake(dataMatrix, 4); // 4 is colIndex for awake in minutes
  println("getting sorted Earliest matrix...");
  dataEarliestCout = getColByAwake(dataMatrix, 5); // 5 is colIndex for earliest cout hour
  println("getting sorted Latest matrix...");
  dataLatestCout = getColByAwake(dataMatrix, 6); // 6 is colIndex for latest cout hour

  for ( int  i = 0; i<  numWeekdays; i++) 
  {
    int[] weekDaysToCenter = null;
    for ( int  j = 0; j < numWeeks; j++)       
    {     
      print(weekDaysToCenter[j]);
      print(dataWeekdayByAwake[i][j]);
      weekDaysToCenter[j] = dataWeekdayByAwake[i][j];//take row and put into 1D array
    }
    //sort values on center
    int weekDaysCentered[] = null;
    weekDaysCentered = sortMaxOnCenterByRow(weekDaysToCenter);
    for ( int  j = 0; j < numWeekdays; j++)       
    { 
      dataWeekdayByAwakeCentered[i][j] = weekDaysCentered[j];//re-transcribe into a matrix
    }
  }
}

// Mapping function to convert SQL output to a matrix that is ordered by AWAKE values, starting in the upper left corner
// values in original data set are already ordered by awake minutes so we can just go through them row by row and write out the weekday and week information
// This look up table could be generated much more efficiently, but given the size of the dataset it works OK.
int[][] getColByAwake(float[][]data, int colIndex) {
  int[][] dataByAwakeMatrix = new int[numWeeks+1][numWeekdays+1];
  //dataByAwakeMatrix[w-1][wd-1] = 0;//zero initializer
  for (int i=0; i<data.length; i++) {//go through values in data file and sort into week weekday matrix
    if ((i != 0) && ((i % numWeekdays) == 0)) {
      println();
    }
    //print(i + " ");
    //print(i / (numWeekdays));
    dataByAwakeMatrix[i%(numWeeks)][i / (numWeeks)] = (int)data[i][colIndex];
    print( dataByAwakeMatrix[i%(numWeeks)][i / (numWeeks)] + " ");
    //dataByAwakeMatrix[i/(numWeekdays)][i % (numWeekdays)] = (int)data[i][colIndex];
    //print( dataByAwakeMatrix[i/(numWeekdays)][i % (numWeekdays)] + " ");
  }
  println();
  return dataByAwakeMatrix;
}

// Interpolation function for missing values in data matrix
int[][] interpMissing(int[][]data) {
  for (int w=1; w<=numWeeks; w++) {
    for (int wd=1; wd<=numWeekdays; wd++) {   
      if (data[w-1][wd-1] == 0) {
        println("zero value detected, interpolate with data for the same day from rest of the year");
        int wdaySum = 0;
        for (int w1=1; w1<=numWeeks; w1++) {
          wdaySum += data[w1-1][wd-1];
        }
        data[w-1][wd-1] = wdaySum/numWeeks;
      }
      print( data[w-1][wd-1] + " ");
    }
    println();      // switch to next line in the prompt, improves legibility
  }
  return data;
}

int[] sortMaxOnCenterByRow(int[]data) {
  int[] n = null;
  for (int i = numWeeks-1; i >=numWeeks; i--) {
    n[i] = i;
  }
  int[] sorted = new int[n.length];

  final int center = (n.length / 2) - 1 + (n.length % 2);
  for (int i = 1, sgn = -1; i <= n.length; i++, sgn *= -1) {
    sorted[center + (sgn*i/2)] = n[i-1];
  }
  return sorted;
}

